package com.kuyuntech.cofarcooking.device.sdk.eventbus.listener;

import com.kuyuntech.cofarcooking.device.sdk.constant.core.CmdTypes;
import com.kuyuntech.cofarcooking.device.sdk.constant.core.PotStatus;
import com.kuyuntech.cofarcooking.device.sdk.eventbus.core.DevInfo;
import com.kuyuntech.cofarcooking.device.sdk.eventbus.event.Dev2McuDataEvent;
import com.kuyuntech.cofarcooking.device.sdk.eventbus.event.DevStatusEvent;
import com.kuyuntech.cofarcooking.device.sdk.eventbus.event.Mcu2DevEvent;
import com.kuyuntech.cofarcooking.device.sdk.protocol.core.BaseDecoder;
import com.kuyuntech.cofarcooking.device.sdk.protocol.core.BaseEncoder;
import com.kuyuntech.cofarcooking.device.sdk.protocol.core.cmd.*;
import com.kuyuntech.cofarcooking.device.sdk.protocol.core.resp.*;
import com.kuyuntech.cofarcooking.device.sdk.util.core.CofarSDK;
import com.kuyuntech.cofarcooking.device.sdk.util.core.Converter;
import org.apache.commons.lang3.StringUtils;
import org.greenrobot.eventbus.Subscribe;

public class Mcu2DevEventListener {

    @Subscribe
    public void onDevStatusEvent(DevStatusEvent event){

    }



    @Subscribe
    public void onMcu2DevEvent(Mcu2DevEvent event) {

        try {
            BaseDecoder baseDecoder = new BaseDecoder(event.getData());


            if (CmdTypes.SHAKE_HANDS.equals(baseDecoder.getProtocol())) {
                ShakeHandsResp shakeHandsResp = new ShakeHandsResp();
                CofarSDK.post2Mcu(shakeHandsResp);
            }

            if (CmdTypes.HEARTBEAT.equals(baseDecoder.getProtocol())) {
                HeartbeatCmd heartbeatCmd = new HeartbeatCmd(baseDecoder);
                DevInfo devInfo = CofarSDK.devInfo();
                devInfo.setMotorDirection(Converter.hexToByte(heartbeatCmd.getMotorDirection()));
                devInfo.setMotorGear(Converter.hexToByte(heartbeatCmd.getMotorGear()));
                devInfo.setAutoGear(Converter.hexToByte(heartbeatCmd.getAutoGear()));
                devInfo.setHeatMode(Converter.hexToByte(heartbeatCmd.getHeatMode()));




                //温度显示逻辑
                short temp = Converter.hexToShort(heartbeatCmd.getTemp());
                if(temp < 37 && !devInfo.isArrive37Temp()){
                     temp = 0;
                }else if (temp < 37 && devInfo.isArrive37Temp()){
                    temp = 37;
                }

                if(temp >= 37){
                    devInfo.setArrive37Temp(true);
                }

                devInfo.setTemp(CofarSDK.parseTemp(temp));
                devInfo.setRealTemp(Converter.hexToShort(heartbeatCmd.getTemp()));
                devInfo.setTargetTemp(CofarSDK.parseTemp(Converter.hexToShort(heartbeatCmd.getTargetTemp())));
//                devInfo.setRemainTime(Converter.hexToShort(heartbeatCmd.getRemainTime()));
//                devInfo.setTargetTime(Converter.hexToShort(heartbeatCmd.getTargetTime()));
                devInfo.setPotStatus(Converter.hexToByte(heartbeatCmd.getPotStatus()));

                if(PotStatus.UNEXIST == devInfo.getPotStatus()){
                      devInfo.setArrive37Temp(false);
                }

                devInfo.setPotCloverStatus(Converter.hexToByte(heartbeatCmd.getPotCloverStatus()));
                devInfo.setErrCode(Converter.hexToByte(heartbeatCmd.getErrCode()));
                devInfo.setResetZero(Converter.hexToByte(heartbeatCmd.getResetZero()));
                devInfo.setRunningStatus(Converter.hexToByte(heartbeatCmd.getRunningStatus()));
                //devInfo.setStatus(devInfo.getRunningStatus());
                devInfo.setWeightStatus(false);
                devInfo.setPotCloverEleStatus(Converter.hexToByte(heartbeatCmd.getPotCloverEleStatus()));
                if(StringUtils.isNotEmpty(heartbeatCmd.getPotCoverType())) devInfo.setPotCloverType(Converter.hexToByte(heartbeatCmd.getPotCoverType()));
                if(StringUtils.isNotEmpty(heartbeatCmd.getWaterSprayGear())) devInfo.setWaterSprayGear(Converter.hexToByte(heartbeatCmd.getWaterSprayGear()));
                if(StringUtils.isNotEmpty(heartbeatCmd.getSupportWaterSpray()))  devInfo.setSupportWaterSpray(Converter.hexToByte(heartbeatCmd.getSupportWaterSpray()));
                CofarSDK.updateDevInfo(devInfo);
            }

            if (CmdTypes.WEIGHT_DATA.equals(baseDecoder.getProtocol())) {
                WeightDataCmd cmd = new WeightDataCmd(baseDecoder);
                int weight = CofarSDK.parseWeight(Converter.hexToInt(cmd.getWeight()));

                if(weight > Short.MAX_VALUE){
                     weight = Short.MAX_VALUE;
                }

                byte type = Converter.hexToByte(cmd.getType());
                CofarSDK.updateWeightData(type == 0 ? (short) weight : (short) -weight);
            }

            if (CmdTypes.WEIGHT_ALIGN_DATA.equals(baseDecoder.getProtocol())) {
                WeightAlignDataCmd cmd = new WeightAlignDataCmd(baseDecoder);
                CofarSDK.updateWeightAlignData(Converter.hexToByte(cmd.getAlignStatus()),Converter.hexToByte(cmd.getAlignPoint()));
            }

            if (CmdTypes.SHAKE_HANDS.equals(baseDecoder.getProtocol())) {
                ShakeHandsCmd cmd = new ShakeHandsCmd(baseDecoder);
                CofarSDK.updateVersionInfo(
                        Converter.hexToShort(cmd.getManufacturer()),
                        Converter.hexToShort(cmd.getHaradware()),
                        Converter.hexToShort(cmd.getSoftware()),
                        Converter.hexToByte(cmd.getProtocol()),
                        Converter.hexToByte(cmd.getDevType())
                );
            }

            if (CmdTypes.VERION_QUERY.equals(baseDecoder.getProtocol())) {
                VersionQueryResp cmd = new VersionQueryResp(baseDecoder);
                CofarSDK.updateVersionInfo(
                        Converter.hexToShort(cmd.getManufacturer()),
                        Converter.hexToShort(cmd.getHaradware()),
                        Converter.hexToShort(cmd.getSoftware()),
                        Converter.hexToByte(cmd.getProtocol()),
                        Converter.hexToByte(cmd.getDevType())
                );
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

}
