package com.kuyuntech.cofarcooking.device.sdk.protocol.core;


import com.kuyuntech.cofarcooking.device.sdk.constant.core.CmdTypes;
import com.kuyuntech.cofarcooking.device.sdk.protocol.core.cmd.*;
import com.kuyuntech.cofarcooking.device.sdk.protocol.core.resp.*;
import com.kuyuntech.cofarcooking.device.sdk.util.core.Converter;
import lombok.Data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Data
public class BaseDecoder {

    //源数据
    private byte[] data ;

    private BaseBody baseBody;

    private Object obj;


    private Class objClass;

    private String protocol;

    public void init(){

        if(this.data[0] != 0x7e || this.data[this.data.length - 1] != 0x7e){
            throw new RuntimeException("帧头或帧尾不符合数据规范");
        }


        //反转义
        byte[] tmp = Arrays.copyOfRange(this.data,1,this.data.length - 1);
        List<Byte> srcByteList = new ArrayList<Byte>();

        //反转义
        for (int i = 0; i < tmp.length; i++) {
            if(tmp[i] == 0x7d && i < (tmp.length - 1) && tmp[i+1] == 0x02){
                srcByteList.add((byte) 0x7e);
                i++;
                continue;
            }

            if(tmp[i] == 0x7d && i < (tmp.length - 1)  && tmp[i+1] == 0x01){
                srcByteList.add((byte) 0x7d);
                i++;
                continue;
            }
            srcByteList.add(tmp[i]);
        }

        byte[] srcBytes = new byte[srcByteList.size()];
        for (int i = 0; i < srcByteList.size(); i++) {
            srcBytes[i] = srcByteList.get(i);
        }

        //校验和
        int sum = 0;
        for (int i = 0; i < (srcBytes.length - 1) ; i++) {
            sum +=srcBytes[i];
        }

        if((((byte) sum) != srcBytes[srcBytes.length - 1])){
            throw new RuntimeException("验证数据校验和失败");
        }

        if(( srcBytes[0] != (srcBytes.length - 2 + 4))){
            throw new RuntimeException("验证数据长度失败");

        }

        this.baseBody = new BaseBody(Arrays.copyOfRange(srcBytes,1,srcBytes.length -1));

        String contentStr = Converter.bytesToHexString(this.baseBody.getContent(),this.baseBody.getContent().length);

        if(contentStr != null &&  !"".equals(contentStr)){




            if(contentStr.startsWith(CmdTypes.CFG_BEEP)){
                this.obj = new CfgBeepResp(this);
                this.protocol = CmdTypes.CFG_BEEP;
                this.objClass = CfgBeepResp.class;
            }

            if(contentStr.startsWith(CmdTypes.CFG_HEAT)){
                this.obj = new CfgHeatResp(this);
                this.protocol = CmdTypes.CFG_BEEP;
                this.objClass = CfgHeatResp.class;
            }


            if(contentStr.startsWith(CmdTypes.CFG_TIME)){
                this.obj = new CfgTimeResp(this);
                this.protocol = CmdTypes.CFG_TIME;
                this.objClass = CfgTimeResp.class;
            }


            if(contentStr.startsWith(CmdTypes.CFG_MOTOR)){
                this.obj = new CfgMotorResp(this);
                this.protocol = CmdTypes.CFG_MOTOR;
                this.objClass = CfgMotorResp.class;
            }


            if(contentStr.startsWith(CmdTypes.HEARTBEAT)){
                this.obj = new HeartbeatCmd(this);
                this.protocol = CmdTypes.HEARTBEAT;
                this.objClass = HeartbeatCmd.class;
            }

            if(contentStr.startsWith(CmdTypes.JOG)){
                this.obj = new CfgJogResp(this);
                this.protocol = CmdTypes.JOG;
                this.objClass = CfgJogResp.class;
            }

            if(contentStr.startsWith(CmdTypes.MCU_RESET)){
                this.obj = new McuRestCmd(this);
                this.protocol = CmdTypes.MCU_RESET;
                this.objClass = McuRestCmd.class;
            }

            if(contentStr.startsWith(CmdTypes.PAUSE)){
                this.obj = new PauseResp(this);
                this.protocol = CmdTypes.PAUSE;
                this.objClass = PauseResp.class;
            }

            if(contentStr.startsWith(CmdTypes.SHAKE_HANDS)){
                this.obj = new ShakeHandsCmd(this);
                this.protocol = CmdTypes.SHAKE_HANDS;
                this.objClass = ShakeHandsCmd.class;
            }

            if(contentStr.startsWith(CmdTypes.START)){
                this.obj = new StartResp(this);
                this.protocol = CmdTypes.START;
                this.objClass = StartResp.class;
            }

            if(contentStr.startsWith(CmdTypes.STOP)){
                this.obj = new StopResp(this);
                this.protocol = CmdTypes.STOP;
                this.objClass = StopResp.class;
            }

            if(contentStr.startsWith(CmdTypes.SUPER_STOP)){
                this.obj = new SuperStopResp(this);
                this.protocol = CmdTypes.SUPER_STOP;
                this.objClass = SuperStopResp.class;
            }

            if(contentStr.startsWith(CmdTypes.VERION_QUERY)){
                this.obj = new VersionQueryResp(this);
                this.protocol = CmdTypes.VERION_QUERY;
                this.objClass = VersionQueryResp.class;
            }

            if(contentStr.startsWith(CmdTypes.WEIGHT_ALIGN_DATA)){
                this.obj = new WeightAlignDataCmd(this);
                this.protocol = CmdTypes.WEIGHT_ALIGN_DATA;
                this.objClass = WeightAlignDataCmd.class;
            }

            if(contentStr.startsWith(CmdTypes.WEIGHT_ALIGN_START)){
                this.obj = new WeightAlignStartResp(this);
                this.protocol = CmdTypes.WEIGHT_ALIGN_START;
                this.objClass = WeightAlignStartResp.class;
            }

            if(contentStr.startsWith(CmdTypes.WEIGHT_ALIGN_STOP)){
                this.obj = new WeightAlignStopResp(this);
                this.protocol = CmdTypes.WEIGHT_ALIGN_STOP;
                this.objClass = WeightAlignStopResp.class;
            }

            if(contentStr.startsWith(CmdTypes.WEIGHT_CLEAR)){
                this.obj = new WeightClearResp(this);
                this.protocol = CmdTypes.WEIGHT_CLEAR;
                this.objClass = WeightClearResp.class;
            }

            if(contentStr.startsWith(CmdTypes.WEIGHT_DATA)){
                this.obj = new WeightDataResp(this);
                this.protocol = CmdTypes.WEIGHT_DATA;
                this.objClass = WeightDataResp.class;
            }

            if(contentStr.startsWith(CmdTypes.WEIGHT_START)){
                this.obj = new WeightStartResp(this);
                this.protocol = CmdTypes.WEIGHT_START;
                this.objClass = WeightStartResp.class;
            }

            if(contentStr.startsWith(CmdTypes.WEIHT_STOP)){
                this.obj = new WeightStartResp(this);
                this.protocol = CmdTypes.WEIHT_STOP;
                this.objClass = WeightStartResp.class;
            }


        }



    }

    public BaseDecoder(byte[] data){
        this.data = data;
        this.init();
    }

    public BaseDecoder(String data){
        this.data = Converter.hexStringToBytes(data);
        this.init();
    }

    public byte[] escapeBytes(){
        return this.baseBody.escapeBytes();
    }

    public String escapeBytesStr(){
        return this.baseBody.escapeBytesStr();

    }


    public  byte[] bytes(){
        return this.baseBody.bytes();

    }

    public  String bytesStr(){
       return this.baseBody.bytesStr();
    }


    public byte[] getData() {
        return data;
    }

    public void setData(byte[] data) {
        this.data = data;
    }

    public BaseBody getBaseBody() {
        return baseBody;
    }

    public void setBaseBody(BaseBody baseBody) {
        this.baseBody = baseBody;
    }

    public Object getObj() {
        return obj;
    }

    public void setObj(Object obj) {
        this.obj = obj;
    }

    public String getProtocol() {
        return protocol;
    }

    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }


    public static void main(String[] args){
        System.out.println(new CfgMotorCmd((byte) 0,(byte) 1,(byte) 1).escapeBytesStr());
        System.out.println(new StartCmd().escapeBytesStr());
    }

}
